library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity encrypt is
    Port (
	        reset : in std_logic;
           clk : in std_logic;
           load_key : in std_logic;
           key : in std_logic_vector(7 downto 0);
           plaintext_1,plaintext_2,plaintext_3,plaintext_4,plaintext_5,plaintext_6,plaintext_7,plaintext_8 : in std_logic_vector(7 downto 0);
           ciphertext_1,ciphertext_2,ciphertext_3,ciphertext_4,ciphertext_5,ciphertext_6,ciphertext_7,ciphertext_8 : out std_logic_vector(7 downto 0);
			  subkeys_ready : out std_logic
			  );
end encrypt;

architecture behavioral of encrypt is

component key_gen
    Port ( reset : in std_logic;
           clk : in std_logic;
			  load : in std_logic;
           key_in : in std_logic_vector(7 downto 0);
           k1_1,k1_2,k1_3,k1_4,k1_5,k1_6,k1_7,k1_8 : out std_logic_vector(7 downto 0);
           k2_1,k2_2,k2_3,k2_4,k2_5,k2_6,k2_7,k2_8 : out std_logic_vector(7 downto 0);
           k3_1,k3_2,k3_3,k3_4,k3_5,k3_6,k3_7,k3_8 : out std_logic_vector(7 downto 0);
           k4_1,k4_2,k4_3,k4_4,k4_5,k4_6,k4_7,k4_8 : out std_logic_vector(7 downto 0);
           k5_1,k5_2,k5_3,k5_4,k5_5,k5_6,k5_7,k5_8 : out std_logic_vector(7 downto 0);
           k6_1,k6_2,k6_3,k6_4,k6_5,k6_6,k6_7,k6_8 : out std_logic_vector(7 downto 0);
           k7_1,k7_2,k7_3,k7_4,k7_5,k7_6,k7_7,k7_8 : out std_logic_vector(7 downto 0);
           k8_1,k8_2,k8_3,k8_4,k8_5,k8_6,k8_7,k8_8 : out std_logic_vector(7 downto 0);
           k9_1,k9_2,k9_3,k9_4,k9_5,k9_6,k9_7,k9_8 : out std_logic_vector(7 downto 0);
           k10_1,k10_2,k10_3,k10_4,k10_5,k10_6,k10_7,k10_8 : out std_logic_vector(7 downto 0);
           k11_1,k11_2,k11_3,k11_4,k11_5,k11_6,k11_7,k11_8 : out std_logic_vector(7 downto 0);
           k12_1,k12_2,k12_3,k12_4,k12_5,k12_6,k12_7,k12_8 : out std_logic_vector(7 downto 0);
           k13_1,k13_2,k13_3,k13_4,k13_5,k13_6,k13_7,k13_8 : out std_logic_vector(7 downto 0);
			  keys_ready:out std_logic
			  );
end component;

component round_function
    Port ( clk : in std_logic;
	        x1,x2,x3,x4,x5,x6,x7,x8 : in std_logic_vector(7 downto 0);
           k1,k2,k3,k4,k5,k6,k7,k8 : in std_logic_vector(7 downto 0);
           k9,k10,k11,k12,k13,k14,k15,k16 : in std_logic_vector(7 downto 0);
           y1,y2,y3,y4,y5,y6,y7,y8 : out std_logic_vector(7 downto 0)
           			  );
end component;

component output_transformation
    Port ( clk : in std_logic;
	        x1,x2,x3,x4,x5,x6,x7,x8 : in std_logic_vector(7 downto 0);
           k1,k2,k3,k4,k5,k6,k7,k8 : in std_logic_vector(7 downto 0);
           y1,y2,y3,y4,y5,y6,y7,y8 : out std_logic_vector(7 downto 0)
			  );
end component;

signal keys_ready : std_logic;
signal x1,x2,x3,x4,x5,x6,x7,x8 : std_logic_vector(7 downto 0);
signal k1_1,k1_2,k1_3,k1_4,k1_5,k1_6,k1_7,k1_8 :  std_logic_vector(7 downto 0);
signal k2_1,k2_2,k2_3,k2_4,k2_5,k2_6,k2_7,k2_8 :  std_logic_vector(7 downto 0);
signal k3_1,k3_2,k3_3,k3_4,k3_5,k3_6,k3_7,k3_8 :  std_logic_vector(7 downto 0);
signal k4_1,k4_2,k4_3,k4_4,k4_5,k4_6,k4_7,k4_8 :  std_logic_vector(7 downto 0);
signal k5_1,k5_2,k5_3,k5_4,k5_5,k5_6,k5_7,k5_8 :  std_logic_vector(7 downto 0);
signal k6_1,k6_2,k6_3,k6_4,k6_5,k6_6,k6_7,k6_8 :  std_logic_vector(7 downto 0);
signal k7_1,k7_2,k7_3,k7_4,k7_5,k7_6,k7_7,k7_8 :  std_logic_vector(7 downto 0);
signal k8_1,k8_2,k8_3,k8_4,k8_5,k8_6,k8_7,k8_8 :  std_logic_vector(7 downto 0);
signal k9_1,k9_2,k9_3,k9_4,k9_5,k9_6,k9_7,k9_8 :  std_logic_vector(7 downto 0);
signal k10_1,k10_2,k10_3,k10_4,k10_5,k10_6,k10_7,k10_8 :  std_logic_vector(7 downto 0);
signal k11_1,k11_2,k11_3,k11_4,k11_5,k11_6,k11_7,k11_8 :  std_logic_vector(7 downto 0);
signal k12_1,k12_2,k12_3,k12_4,k12_5,k12_6,k12_7,k12_8 :  std_logic_vector(7 downto 0);
signal k13_1,k13_2,k13_3,k13_4,k13_5,k13_6,k13_7,k13_8 :  std_logic_vector(7 downto 0);
signal y1_1,y1_2,y1_3,y1_4,y1_5,y1_6,y1_7,y1_8 :  std_logic_vector(7 downto 0);
signal y2_1,y2_2,y2_3,y2_4,y2_5,y2_6,y2_7,y2_8 :  std_logic_vector(7 downto 0);
signal y3_1,y3_2,y3_3,y3_4,y3_5,y3_6,y3_7,y3_8 :  std_logic_vector(7 downto 0);
signal y4_1,y4_2,y4_3,y4_4,y4_5,y4_6,y4_7,y4_8 :  std_logic_vector(7 downto 0);
signal y5_1,y5_2,y5_3,y5_4,y5_5,y5_6,y5_7,y5_8 :  std_logic_vector(7 downto 0);
signal y6_1,y6_2,y6_3,y6_4,y6_5,y6_6,y6_7,y6_8 :  std_logic_vector(7 downto 0);
signal y7_1,y7_2,y7_3,y7_4,y7_5,y7_6,y7_7,y7_8 :  std_logic_vector(7 downto 0);

begin

key_generation_component:key_gen port map(
reset,clk,load_key,key,
k1_1,k1_2,k1_3,k1_4,k1_5,k1_6,k1_7,k1_8,
k2_1,k2_2,k2_3,k2_4,k2_5,k2_6,k2_7,k2_8,
k3_1,k3_2,k3_3,k3_4,k3_5,k3_6,k3_7,k3_8,
k4_1,k4_2,k4_3,k4_4,k4_5,k4_6,k4_7,k4_8,
k5_1,k5_2,k5_3,k5_4,k5_5,k5_6,k5_7,k5_8,
k6_1,k6_2,k6_3,k6_4,k6_5,k6_6,k6_7,k6_8,
k7_1,k7_2,k7_3,k7_4,k7_5,k7_6,k7_7,k7_8,
k8_1,k8_2,k8_3,k8_4,k8_5,k8_6,k8_7,k8_8,
k9_1,k9_2,k9_3,k9_4,k9_5,k9_6,k9_7,k9_8,
k10_1,k10_2,k10_3,k10_4,k10_5,k10_6,k10_7,k10_8,
k11_1,k11_2,k11_3,k11_4,k11_5,k11_6,k11_7,k11_8,
k12_1,k12_2,k12_3,k12_4,k12_5,k12_6,k12_7,k12_8,
k13_1,k13_2,k13_3,k13_4,k13_5,k13_6,k13_7,k13_8,
keys_ready);

process(clk)
begin
if (clk'event and clk='1') then
  if keys_ready='1' then
    x1<=plaintext_1;
    x2<=plaintext_2;
    x3<=plaintext_3;
    x4<=plaintext_4;
	 x5<=plaintext_5;
	 x6<=plaintext_6;
	 x7<=plaintext_7;
	 x8<=plaintext_8;
  end if;
end if;
end process;

round_1:round_function port map
        (clk,x1,x2,x3,x4,x5,x6,x7,x8,
		  k1_1,k1_2,k1_3,k1_4,k1_5,k1_6,k1_7,k1_8,
		  k2_1,k2_2,k2_3,k2_4,k2_5,k2_6,k2_7,k2_8,
		  y1_1,y1_2,y1_3,y1_4,y1_5,y1_6,y1_7,y1_8);

round_2:round_function port map
        (clk,y1_1,y1_2,y1_3,y1_4,y1_5,y1_6,y1_7,y1_8,
		  k3_1,k3_2,k3_3,k3_4,k3_5,k3_6,k3_7,k3_8,
		  k4_1,k4_2,k4_3,k4_4,k4_5,k4_6,k4_7,k4_8,
		  y2_1,y2_2,y2_3,y2_4,y2_5,y2_6,y2_7,y2_8);			
			
round_3:round_function port map
        (clk,y2_1,y2_2,y2_3,y2_4,y2_5,y2_6,y2_7,y2_8,
		  k5_1,k5_2,k5_3,k5_4,k5_5,k5_6,k5_7,k5_8,
		  k6_1,k6_2,k6_3,k6_4,k6_5,k6_6,k6_7,k6_8,
		  y3_1,y3_2,y3_3,y3_4,y3_5,y3_6,y3_7,y3_8);			

round_4:round_function port map
        (clk,y3_1,y3_2,y3_3,y3_4,y3_5,y3_6,y3_7,y3_8,
		  k7_1,k7_2,k7_3,k7_4,k7_5,k7_6,k7_7,k7_8,
		  k8_1,k8_2,k8_3,k8_4,k8_5,k8_6,k8_7,k8_8,
		  y4_1,y4_2,y4_3,y4_4,y4_5,y4_6,y4_7,y4_8);		  
		  
round_5:round_function port map
        (clk,y4_1,y4_2,y4_3,y4_4,y4_5,y4_6,y4_7,y4_8,
		  k9_1,k9_2,k9_3,k9_4,k9_5,k9_6,k9_7,k9_8,
		  k10_1,k10_2,k10_3,k10_4,k10_5,k10_6,k10_7,k10_8,
		  y5_1,y5_2,y5_3,y5_4,y5_5,y5_6,y5_7,y5_8);		  

round_6:round_function port map
        (clk,y5_1,y5_2,y5_3,y5_4,y5_5,y5_6,y5_7,y5_8,
		  k11_1,k11_2,k11_3,k11_4,k11_5,k11_6,k11_7,k11_8,
		  k12_1,k12_2,k12_3,k12_4,k12_5,k12_6,k12_7,k12_8,
		  y6_1,y6_2,y6_3,y6_4,y6_5,y6_6,y6_7,y6_8);
		  
out_transform:output_transformation port map
          (clk,y6_1,y6_2,y6_3,y6_4,y6_5,y6_6,y6_7,y6_8,
	        k13_1,k13_2,k13_3,k13_4,k13_5,k13_6,k13_7,k13_8,
           y7_1,y7_2,y7_3,y7_4,y7_5,y7_6,y7_7,y7_8);		  


process(clk)
begin
if (clk'event and clk='1') then
  ciphertext_1<=y7_1;
  ciphertext_2<=y7_2;
  ciphertext_3<=y7_3;
  ciphertext_4<=y7_4;
  ciphertext_5<=y7_5;
  ciphertext_6<=y7_6;
  ciphertext_7<=y7_7;
  ciphertext_8<=y7_8;
end if;
end process;
			  
subkeys_ready <= keys_ready;


end behavioral;
--<copyright-disclaimer-start>
--  **************************************************************************************************************
--  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
--  * DISCLAIMER                                                                                                 *
--  * The information contained herein is for informational purposes only, and is subject to change              *
--  * without notice. While every precaution has been taken in the preparation of this document, it              *
--  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
--  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
--  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
--  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
--  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
--  * hardware, software or other products described herein.  No license, including implied or                   *
--  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
--  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
--  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
--  *                                                                                                            *
--  **************************************************************************************************************
--<copyright-disclaimer-end>