
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity round_function is
    Port ( clk : in std_logic;
	        x1,x2,x3,x4,x5,x6,x7,x8 : in std_logic_vector(7 downto 0);
           k1,k2,k3,k4,k5,k6,k7,k8 : in std_logic_vector(7 downto 0);
           k9,k10,k11,k12,k13,k14,k15,k16 : in std_logic_vector(7 downto 0);
           y1,y2,y3,y4,y5,y6,y7,y8 : out std_logic_vector(7 downto 0)
           			  );
end round_function;

architecture behavioral of round_function is

component exp_log_table
    Port (
	Exp_In:     IN     STD_LOGIC_VECTOR (8 DOWNTO 0);    
	Log_In:     IN     STD_LOGIC_VECTOR (8 DOWNTO 0);    
	Clk:        IN     STD_LOGIC;    
	Exp_Out:       OUT    STD_LOGIC_VECTOR (7 DOWNTO 0);    
	Log_Out:       OUT    STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
end component;

component two_pht
    Port ( clk : in std_logic;
	 		  a1 : in std_logic_vector(7 downto 0);
           a2 : in std_logic_vector(7 downto 0);
           b1 : out std_logic_vector(7 downto 0);
           b2 : out std_logic_vector(7 downto 0));
end component;

signal a1,a2,a3,a4,a5,a6,a7,a8:STD_LOGIC_VECTOR (7 DOWNTO 0); 
signal b1,b2,b3,b4,b5,b6,b7,b8:STD_LOGIC_VECTOR (7 DOWNTO 0); 
signal c1,c2,c3,c4,c5,c6,c7,c8:STD_LOGIC_VECTOR (7 DOWNTO 0); 
signal d1,d2,d3,d4,d5,d6,d7,d8:STD_LOGIC_VECTOR (7 DOWNTO 0); 
signal e1,e2,e3,e4,e5,e6,e7,e8:STD_LOGIC_VECTOR (7 DOWNTO 0); 

-- the following signals declarations are to work around the fact that XST interprets the VHDL LRM 
-- to say that the expression [Exp_In=>'0'&a1] is not valid in a port map

signal a1_sig, a2_sig, a3_sig, a4_sig, a5_sig, a6_sig, a7_sig, a8_sig:STD_LOGIC_VECTOR (8 DOWNTO 0);

begin 
process(clk)
begin
if (clk'event and clk='1') then
  a1 <= x1 xor k1; a2 <= x2 + k2; a3 <= x3 + k3; a4 <= x4 xor k4;
  a5 <= x5 xor k5; a6 <= x6 + k6; a7 <= x7 + k7; a8 <= x8 xor k8;
end if;
end process;


-- the following signal assignments are to work around the fact that XST interprets the VHDL LRM 
-- to say that the expression [Exp_In=>'0'&a1] is not valid in a port map

a1_sig<='0'&a1;
a2_sig<='1'&a2;
a3_sig<='1'&a3;
a4_sig<='0'&a4;
a5_sig<='0'&a5;
a6_sig<='1'&a6;
a7_sig<='1'&a7;
a8_sig<='0'&a8;

exp_log_table_1:exp_log_table port map
						(Exp_In=>a1_sig,Log_In=>a2_sig,Clk=>clk,Exp_Out=>b1,Log_Out=>b2);
exp_log_table_2:exp_log_table port map
						(Exp_In=>a4_sig,Log_In=>a3_sig,Clk=>clk,Exp_Out=>b4,Log_Out=>b3);
exp_log_table_3:exp_log_table port map
						(Exp_In=>a5_sig,Log_In=>a6_sig,Clk=>clk,Exp_Out=>b5,Log_Out=>b6);
exp_log_table_4:exp_log_table port map
						(Exp_In=>a8_sig,Log_In=>a7_sig,Clk=>clk,Exp_Out=>b8,Log_Out=>b7);

process(clk)
begin
if (clk'event and clk='1') then
  c1 <= b1 + k9; c2 <= b2 xor k10; c3 <= b3 xor k11; c4 <= b4 + k12;
  c5 <= b5 + k13; c6 <= b6 xor k14; c7 <= b7 xor k15; c8 <= b8 + k16;
end if;
end process;					


pht_1_1:two_pht port map(clk,c1,c2,d1,d2);
pht_1_2:two_pht port map(clk,c3,c4,d3,d4);
pht_1_3:two_pht port map(clk,c5,c6,d5,d6);
pht_1_4:two_pht port map(clk,c7,c8,d7,d8);

pht_2_1:two_pht port map(clk,d1,d3,e1,e2);
pht_2_2:two_pht port map(clk,d5,d7,e3,e4);
pht_2_3:two_pht port map(clk,d2,d4,e5,e6);
pht_2_4:two_pht port map(clk,d6,d8,e7,e8);

pht_3_1:two_pht port map(clk,e1,e3,y1,y2);
pht_3_2:two_pht port map(clk,e5,e7,y3,y4);
pht_3_3:two_pht port map(clk,e2,e4,y5,y6);
pht_3_4:two_pht port map(clk,e6,e8,y7,y8);


end behavioral;
--<copyright-disclaimer-start>
--  **************************************************************************************************************
--  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
--  * DISCLAIMER                                                                                                 *
--  * The information contained herein is for informational purposes only, and is subject to change              *
--  * without notice. While every precaution has been taken in the preparation of this document, it              *
--  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
--  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
--  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
--  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
--  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
--  * hardware, software or other products described herein.  No license, including implied or                   *
--  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
--  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
--  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
--  *                                                                                                            *
--  **************************************************************************************************************
--<copyright-disclaimer-end>