`timescale 1ns/1ps

module top_s3e_vs_c2 (

   input CLK0,input RST, input CE,
   output CLK_CNT,
   output [64:0] DATA_OUTS
);

   parameter num_instances = 30;
   parameter num_inputs = 73 + num_instances-1;
   parameter num_outputs = 65;
   parameter clock_bits = 26;

   reg [clock_bits-1:0] clock_count = {clock_bits{1'b0}};
   reg [4:0] data_sel1 = 5'h0;
//  wire RST, CE;
   wire clk_ibufo, clk;
   reg [num_inputs-1:0] data_in_reg = {num_inputs{1'b0}};
   reg [num_outputs-1:0] data_out = {num_outputs{1'b0}};
   wire [(num_outputs*num_instances)-1:0] data_int_o;

   integer j;

//  assign RST = 1'b1;
//  assign CE = 1'b1;
    // synthesis translate_off
/*    initial begin
      force RST = 1'b0;
      #100 release RST;
    end */
    // synthesis translate_on
// IBUFGDS clkBuf (.O(CLK0), .I(CLK0_p), .IB(CLK0_n));

   assign clk_ibufo = CLK0;

// BUFG clk_bufg (.I(clk_ibufo),.O(clk_bufgo));

   assign clk = clk_ibufo;

   genvar i;

   generate
      for (i=0; i<num_instances; i=i+1)
      begin: gen_inst

      encrypt uut (
            .reset(RST),
            .clk(clk),
            .load_key(data_in_reg[0+i]),
            .key(data_in_reg[8+i:1+i]),
            .plaintext_1(data_in_reg[16+i:9+i]),
            .plaintext_2(data_in_reg[24+i:17+i]),
            .plaintext_3(data_in_reg[32+i:25+i]),
            .plaintext_4(data_in_reg[40+i:33+i]),
            .plaintext_5(data_in_reg[48+i:41+i]),
            .plaintext_6(data_in_reg[56+i:49+i]),
            .plaintext_7(data_in_reg[64+i:57+i]),
            .plaintext_8(data_in_reg[72+i:65+i]),
            .ciphertext_1(data_int_o[7+(num_outputs*i):0+(num_outputs*i)]),
            .ciphertext_2(data_int_o[15+(num_outputs*i):8+(num_outputs*i)]),
            .ciphertext_3(data_int_o[23+(num_outputs*i):16+(num_outputs*i)]),
            .ciphertext_4(data_int_o[31+(num_outputs*i):24+(num_outputs*i)]),
            .ciphertext_5(data_int_o[39+(num_outputs*i):32+(num_outputs*i)]),
            .ciphertext_6(data_int_o[47+(num_outputs*i):40+(num_outputs*i)]),
            .ciphertext_7(data_int_o[55+(num_outputs*i):48+(num_outputs*i)]),
            .ciphertext_8(data_int_o[63+(num_outputs*i):56+(num_outputs*i)]),
            .subkeys_ready(data_int_o[64+(num_outputs*i)])
         );

      end
   endgenerate

   always @(posedge clk)
      if (!RST)
         data_out[num_outputs-1:0] <= {num_outputs{1'b0}};
      else if (CE)
         case (data_sel1[4:0])
            5'd1: data_out[num_outputs-1:0] <= data_int_o[(2*num_outputs)-1:num_outputs];
            5'd2: data_out[num_outputs-1:0] <= data_int_o[(3*num_outputs)-1:(2*num_outputs)];
            5'd3: data_out[num_outputs-1:0] <= data_int_o[(4*num_outputs)-1:(3*num_outputs)];
            5'd4: data_out[num_outputs-1:0] <= data_int_o[(5*num_outputs)-1:(4*num_outputs)];
            5'd5: data_out[num_outputs-1:0] <= data_int_o[(6*num_outputs)-1:(5*num_outputs)];
            5'd6: data_out[num_outputs-1:0] <= data_int_o[(7*num_outputs)-1:(6*num_outputs)];
            5'd7: data_out[num_outputs-1:0] <= data_int_o[(8*num_outputs)-1:(7*num_outputs)];
            5'd8: data_out[num_outputs-1:0] <= data_int_o[(9*num_outputs)-1:(8*num_outputs)];
            5'd9: data_out[num_outputs-1:0] <= data_int_o[(10*num_outputs)-1:(9*num_outputs)];
            5'd10: data_out[num_outputs-1:0] <= data_int_o[(11*num_outputs)-1:(10*num_outputs)];
            5'd11: data_out[num_outputs-1:0] <= data_int_o[(12*num_outputs)-1:(11*num_outputs)];
            5'd12: data_out[num_outputs-1:0] <= data_int_o[(13*num_outputs)-1:(12*num_outputs)];
            5'd13: data_out[num_outputs-1:0] <= data_int_o[(14*num_outputs)-1:(13*num_outputs)];
            5'd14: data_out[num_outputs-1:0] <= data_int_o[(15*num_outputs)-1:(14*num_outputs)];
            5'd15: data_out[num_outputs-1:0] <= data_int_o[(16*num_outputs)-1:(15*num_outputs)];
            5'd16: data_out[num_outputs-1:0] <= data_int_o[(17*num_outputs)-1:(16*num_outputs)];
            5'd17: data_out[num_outputs-1:0] <= data_int_o[(18*num_outputs)-1:(17*num_outputs)];
            5'd18: data_out[num_outputs-1:0] <= data_int_o[(19*num_outputs)-1:(18*num_outputs)];
            5'd19: data_out[num_outputs-1:0] <= data_int_o[(20*num_outputs)-1:(19*num_outputs)];
            5'd20: data_out[num_outputs-1:0] <= data_int_o[(21*num_outputs)-1:(20*num_outputs)];
            5'd21: data_out[num_outputs-1:0] <= data_int_o[(22*num_outputs)-1:(21*num_outputs)];
            5'd22: data_out[num_outputs-1:0] <= data_int_o[(23*num_outputs)-1:(22*num_outputs)];
            5'd23: data_out[num_outputs-1:0] <= data_int_o[(24*num_outputs)-1:(23*num_outputs)];
            5'd24: data_out[num_outputs-1:0] <= data_int_o[(25*num_outputs)-1:(24*num_outputs)];
            5'd25: data_out[num_outputs-1:0] <= data_int_o[(26*num_outputs)-1:(25*num_outputs)];
            5'd26: data_out[num_outputs-1:0] <= data_int_o[(27*num_outputs)-1:(26*num_outputs)];
            5'd27: data_out[num_outputs-1:0] <= data_int_o[(28*num_outputs)-1:(27*num_outputs)];
            5'd28: data_out[num_outputs-1:0] <= data_int_o[(29*num_outputs)-1:(28*num_outputs)];
            5'd29: data_out[num_outputs-1:0] <= data_int_o[(30*num_outputs)-1:(29*num_outputs)];
            default: data_out[num_outputs-1:0] <= data_int_o[num_outputs-1:0];
         endcase

   always @(posedge clk)
      if (!RST) begin
         clock_count <= {clock_bits{1'b0}};
         data_sel1 <= 5'h0;
      end
      else if (CE) begin
         clock_count <= clock_count + 1;
         data_sel1 <= data_sel1 + 1;
      end

   // LFSR Input Stimulus (feedback at 2^14)
   always @(posedge clk)
      if (!RST)
         data_in_reg <= {num_inputs{1'b0}};
      else if (CE)
         data_in_reg <= {data_in_reg[num_inputs-2:0], (data_in_reg[13] ~^ data_in_reg[4] ~^ data_in_reg[2] ~^ data_in_reg[0])};

   assign DATA_OUTS[num_outputs-1:0] = data_out[num_outputs-1:0];

   assign CLK_CNT = clock_count[clock_bits-1];

endmodule
//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>