//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>


#pragma once

#include <adf.h>
#include <fir_resampler_graph.hpp>
#include <fir_interpolate_hb_graph.hpp>
#include <dds_mixer_graph.hpp>
#include "resamp_coeff.h"
#include "hb1_coeff.h"
#include "hb2_coeff.h"
#include "hb3_coeff.h"

using namespace adf;

using namespace xf::dsp::aie::fir::resampler;
using namespace xf::dsp::aie::fir::interpolate_hb;
using namespace xf::dsp::aie::mixer::dds_mixer;

// ------------------------------------------------------------
// Resampler Namespace
// ------------------------------------------------------------

namespace resampler {
  static constexpr int  WIN_SIZE = 256;
  typedef cint16                                TT_DATA;
  typedef int16                                 TT_COEFF;
  static constexpr int  TP_FIR_LEN            = 71;     // Total number of taps
  static constexpr int  TP_INTERPOLATE_FACTOR = 3;
  static constexpr int  TP_DECIMATE_FACTOR    = 2;
  static constexpr int  TP_SHIFT              = 15;     // Depends on FXP properties
  static constexpr int  TP_RND                = 5;      // how to set this
  static constexpr int  TP_INPUT_WINDOW_VSIZE = WIN_SIZE;
  static constexpr int  TP_CASC_LEN           = 1;      // # of tiles in a cascade
  static constexpr int  TP_USE_COEFF_RELOAD   = 0;      // Set to 1 to use coefficient reload feature
  static constexpr int  TP_NUM_OUTPUTS        = 1;      // Add optional 2nd output port for windows
  static constexpr int  TP_DUAL_IP            = 0;      // Only used in stream mode (TP_API = 1)
  static constexpr int  TP_API                = 0;      // Set to 1 for stream mode (windows otherwise)
  static constexpr int  TP_SSR                = 1;      // Scale throughput using parallelization
};

// ------------------------------------------------------------
// Interpolate HB1 Namespace
// ------------------------------------------------------------
namespace hb1 {
  static constexpr int  WIN_SIZE = 384;
  typedef cint16                                TT_DATA;
  typedef int16                                 TT_COEFF;
  static constexpr int  TP_FIR_LEN            = 19;     // Total number of taps
  static constexpr int  TP_SHIFT              = 15;     // Depends on FXP properties
  static constexpr int  TP_RND                = 5;      // how to set this
  static constexpr int  TP_INPUT_WINDOW_VSIZE = WIN_SIZE;
  static constexpr int  TP_CASC_LEN           = 1;      // # of tiles in a cascade
  static constexpr int  TP_DUAL_IP            = 0;      // Only used in stream mode (TP_API = 1)
  static constexpr int  TP_USE_COEFF_RELOAD   = 0;      // Set to 1 to use coefficient reload feature
  static constexpr int  TP_NUM_OUTPUTS        = 1;      // Add optional 2nd output port for windows
  static constexpr int  TP_UPSHIFT_CT         = 1;      // Cnter tap coefficient will be treated as any other coefficient
  static constexpr int  TP_API                = 0;      // Set to 1 for stream mode (windows otherwise)
  static constexpr int  TP_SSR                = 1;      // Scale throughput using parallelization
  static constexpr int  TP_PARA_INTERP_POLY   = 1;      // Sets the number of interpolator polyphases
  static constexpr int  TP_SAT                = 1;      // Saturate
};

// ------------------------------------------------------------
// Interpolate HB2 Namespace
// ------------------------------------------------------------
namespace hb2 {
  static constexpr int  WIN_SIZE = 384;
  typedef cint16                                TT_DATA;
  typedef int16                                 TT_COEFF;
  static constexpr int  TP_FIR_LEN            = 11;     // Total number of taps
  static constexpr int  TP_SHIFT              = 15;     // Depends on FXP properties
  static constexpr int  TP_RND                = 5;      // how to set this
  static constexpr int  TP_INPUT_WINDOW_VSIZE = WIN_SIZE;
  static constexpr int  TP_CASC_LEN           = 1;      // # of tiles in a cascade
  static constexpr int  TP_DUAL_IP            = 0;      // Only used in stream mode (TP_API = 1)
  static constexpr int  TP_USE_COEFF_RELOAD   = 0;      // Set to 1 to use coefficient reload feature
  static constexpr int  TP_NUM_OUTPUTS        = 1;      // Add optional 2nd output port for windows
  static constexpr int  TP_UPSHIFT_CT         = 1;      // Cnter tap coefficient will be treated as any other coefficient
  static constexpr int  TP_API                = 0;      // Set to 1 for stream mode (windows otherwise)
  static constexpr int  TP_SSR                = 1;      // Scale throughput using parallelization
  static constexpr int  TP_PARA_INTERP_POLY   = 1;      // Sets the number of interpolator polyphases
  static constexpr int  TP_SAT                = 1;      // Saturate
};

// ------------------------------------------------------------
// Interpolate HB3 Namespace
// ------------------------------------------------------------
namespace hb3 {
  static constexpr int  WIN_SIZE = 384;
  typedef cint16                                TT_DATA;
  typedef int16                                 TT_COEFF;
  static constexpr int  TP_FIR_LEN            = 7;     // Total number of taps
  static constexpr int  TP_SHIFT              = 15;     // Depends on FXP properties
  static constexpr int  TP_RND                = 5;      // how to set this
  static constexpr int  TP_INPUT_WINDOW_VSIZE = WIN_SIZE;
  static constexpr int  TP_CASC_LEN           = 1;      // # of tiles in a cascade
  static constexpr int  TP_DUAL_IP            = 0;      // Only used in stream mode (TP_API = 1)
  static constexpr int  TP_USE_COEFF_RELOAD   = 0;      // Set to 1 to use coefficient reload feature
  static constexpr int  TP_NUM_OUTPUTS        = 1;      // Add optional 2nd output port for windows
  static constexpr int  TP_UPSHIFT_CT         = 1;      // Cnter tap coefficient will be treated as any other coefficient
  static constexpr int  TP_API                = 0;      // Set to 1 for stream mode (windows otherwise)
  static constexpr int  TP_SSR                = 1;      // Scale throughput using parallelization
  static constexpr int  TP_PARA_INTERP_POLY   = 1;      // Sets the number of interpolator polyphases
  static constexpr int  TP_SAT                = 1;      // Saturate
};

// ------------------------------------------------------------
// DDS Mixer Namespace
// ------------------------------------------------------------
namespace mixer0 {
  static constexpr int  WIN_SIZE = 384;
  typedef cint16                                TT_DATA;
  static constexpr int  TP_INPUT_WINDOW_VSIZE = WIN_SIZE;
  static constexpr int  TP_MIXER_MODE         = 1;      // dds plus single data channel mixer
  static constexpr int  TP_API                = 0;      // Set to 1 for stream mode (windows otherwise)
  static constexpr int  TP_SSR                = 1;      // Scale throughput using parallelization
  static constexpr int  TP_RND                = 5;      // how to set this
  static constexpr int  TP_SAT                = 1;      // Saturate
  static constexpr int  TP_USE_PHASE_RELOAD   = 0;      // static phase initialization, defined in dds constructor   
};

// ------------------------------------------------------------
// Top Level Graph
// ------------------------------------------------------------

class duc_graph : public graph {
public:
  // mixer_0_phaseInc calculation 
  // mixer o/p freq. (Fo) = 200 MHz, Fs = 1200 MHz
  // phaseInc = 2^32 *Fo/Fs = 2^32 * 200/1200 = 715827883
  static constexpr unsigned int mixer_0_phaseInc = 715827883;

  static constexpr unsigned int initialPhaseOffset = 0;

private:
  // Filter taps:
  std::vector<resampler::TT_COEFF> m_taps = std::vector<resampler::TT_COEFF>( RESAMP_COEFF );
  std::vector<hb1::TT_COEFF> hb1_taps = std::vector<hb1::TT_COEFF>( HB1_COEFF );
  std::vector<hb2::TT_COEFF> hb2_taps = std::vector<hb2::TT_COEFF>( HB2_COEFF );
  std::vector<hb3::TT_COEFF> hb3_taps = std::vector<hb3::TT_COEFF>( HB3_COEFF );

  // FIR Resampler class:
  using TT_RESAMPLER = fir_resampler_graph<resampler::TT_DATA,resampler::TT_COEFF,resampler::TP_FIR_LEN,resampler::TP_INTERPOLATE_FACTOR,
                                  resampler::TP_DECIMATE_FACTOR,resampler::TP_SHIFT,
                                  resampler::TP_RND,resampler::TP_INPUT_WINDOW_VSIZE,resampler::TP_CASC_LEN,
                                  resampler::TP_USE_COEFF_RELOAD,resampler::TP_NUM_OUTPUTS,resampler::TP_DUAL_IP,
                                  resampler::TP_API,resampler::TP_SSR>;

  // FIR HB1 Interpolator class:
  using TT_FIR_INTERPOLATE_HB1 = fir_interpolate_hb_graph<hb1::TT_DATA,hb1::TT_COEFF,hb1::TP_FIR_LEN,hb1::TP_SHIFT,
                                  hb1::TP_RND,hb1::TP_INPUT_WINDOW_VSIZE,hb1::TP_CASC_LEN,hb1::TP_DUAL_IP, 
                                  hb1::TP_USE_COEFF_RELOAD,hb1::TP_NUM_OUTPUTS,hb1::TP_UPSHIFT_CT,hb1::TP_API,
                                  hb1::TP_SSR,hb1::TP_PARA_INTERP_POLY,hb1::TP_SAT>;

  // FIR HB2 Interpolator class:
  using TT_FIR_INTERPOLATE_HB2 = fir_interpolate_hb_graph<hb2::TT_DATA,hb2::TT_COEFF,hb2::TP_FIR_LEN,hb2::TP_SHIFT,
                                  hb2::TP_RND,hb2::TP_INPUT_WINDOW_VSIZE,hb2::TP_CASC_LEN,hb2::TP_DUAL_IP, 
                                  hb2::TP_USE_COEFF_RELOAD,hb2::TP_NUM_OUTPUTS,hb2::TP_UPSHIFT_CT,hb2::TP_API,
                                  hb2::TP_SSR,hb2::TP_PARA_INTERP_POLY,hb2::TP_SAT>;

  // FIR HB3 Interpolator class:
  using TT_FIR_INTERPOLATE_HB3 = fir_interpolate_hb_graph<hb3::TT_DATA,hb3::TT_COEFF,hb3::TP_FIR_LEN,hb3::TP_SHIFT,
                                  hb3::TP_RND,hb3::TP_INPUT_WINDOW_VSIZE,hb3::TP_CASC_LEN,hb3::TP_DUAL_IP, 
                                  hb3::TP_USE_COEFF_RELOAD,hb3::TP_NUM_OUTPUTS,hb3::TP_UPSHIFT_CT,hb3::TP_API,
                                  hb3::TP_SSR,hb3::TP_PARA_INTERP_POLY,hb3::TP_SAT>;

 // DDS Mixer class:
 using TT_DDS_MIXER = dds_mixer_graph<mixer0::TT_DATA,mixer0::TP_INPUT_WINDOW_VSIZE,mixer0::TP_MIXER_MODE,
                       mixer0::TP_API,mixer0::TP_SSR,mixer0::TP_RND,mixer0::TP_SAT,mixer0::TP_USE_PHASE_RELOAD>; 

public:
  input_plio  duc_i0;
  output_plio duc_o;

  TT_RESAMPLER resampler_0_dut;
  TT_FIR_INTERPOLATE_HB1 hb1_0_dut;
  TT_FIR_INTERPOLATE_HB2 hb2_0_dut;
  TT_FIR_INTERPOLATE_HB3 hb3_0_dut;
  TT_DDS_MIXER mixer_0_dut;  

  duc_graph(void) : resampler_0_dut( m_taps ), hb1_0_dut( hb1_taps ), hb2_0_dut( hb2_taps ), hb3_0_dut( hb3_taps ), mixer_0_dut( mixer_0_phaseInc, initialPhaseOffset )
  {
    duc_i0 =  input_plio::create( "PLIO_duc_i0", plio_64_bits, "data/input_sig_i0.txt" );
    duc_o  = output_plio::create( "PLIO_duc_o",  plio_64_bits, "data/sig_o.txt" );
    
    // Signal 0
    connect<>( duc_i0.out[0],  resampler_0_dut.in[0] );
    connect<>( resampler_0_dut.out[0], hb1_0_dut.in[0] );
    connect<>( hb1_0_dut.out[0], hb2_0_dut.in[0] );
    connect<>( hb2_0_dut.out[0], hb3_0_dut.in[0] );
    connect<>( hb3_0_dut.out[0], mixer_0_dut.in1[0] );
    connect<>( mixer_0_dut.out[0], duc_o.in[0] );
  }

};
