/*
Copyright (C) 2025, Advanced Micro Devices, Inc. All rights reserved.
SPDX-License-Identifier: MIT
*/





//   DSP48E2   : In order to incorporate this function into the design,
//   Verilog   : the following instance declaration needs to be placed
//  instance   : in the body of the design code.  The instance name
// declaration : (DSP48E2_inst) and/or the port declarations within the
//    code     : parenthesis may be changed to properly reference and
//             : connect this function to the design.  All inputs
//             : and outputs must be connected.

//  <-----Cut code below this line---->

   // DSP48E2: 48-bit Multi-Functional Arithmetic Block
   //          Virtex UltraScale
   // Xilinx HDL Language Template, version 2024.1

module DSP58_top #(
  parameter TRUE = 1,
  parameter FALSE = 0,
  parameter REGISTER_Y    = TRUE,  // Enable DSP PREG: 0=bypass PREG; 1=use PREG
  parameter P1   = TRUE,  // Input pipeline depth. 
  parameter P2   = TRUE,  // Internal pipeline depth. 
  parameter P3   = TRUE,  // Another internal pipeline depth. 
  parameter A_DATA_WIDTH  = 27,    // A input data width
  parameter DSP_A_WIDTH = 27,
  parameter SIGN_EXTEND_A = 0, // Sign extend A to DSP A port; only when A_DATA_WIDTH < DSP A port width
  parameter B_DATA_WIDTH  = 24,    // B input data width.
  parameter DSP_B_WIDTH = 24,
  parameter SIGN_EXTEND_B = 0, // Sign extend B to DSP B port; only when B_DATA_WIDTH < DSP B port width
  parameter SIGN_EXTEND_C = 0,  // sign extend C to DSP C port; only when C_DATA_WIDTH < DSP C port width
  parameter Y_DATA_WIDTH  = 58    // DSP output data width
)  (

  input                     clk,

  input                     b_wen,     // clock enable A ping register 
  input  [A_DATA_WIDTH-1:0] a_data,      // A data input
  input  [B_DATA_WIDTH-1:0] b_data,      // B data input
  input  [8:0]              OPMODE, 
  input  [Y_DATA_WIDTH-1:0] y_data_in_cascade,  // Vertical input data for summation
  output [Y_DATA_WIDTH-1:0] y_data_out_cascade, // Vertical sum output, dedicated DSP cascade
  output [Y_DATA_WIDTH-1:0] y_data_out_fabric   // DSP output to fabric = DSP P output

  );

  wire [DSP_A_WIDTH-1:0] dsp_a_data = SIGN_EXTEND_A ? $signed(a_data) : $unsigned(a_data);
  wire [DSP_B_WIDTH-1:0] dsp_b_data = SIGN_EXTEND_B ? $signed(b_data) : $unsigned(b_data);
  wire use_c_data = 1'b0;


   DSP48E2 #(
      // Feature Control Attributes: Data Path Selection
      .AMULTSEL("A"),                    // Selects A input to multiplier (A, AD)
      .A_INPUT("DIRECT"),                // Selects A input source, "DIRECT" (A port) or "CASCADE" (ACIN port)
      .BMULTSEL("B"),                    // Selects B input to multiplier (AD, B)
      .B_INPUT("DIRECT"),                // Selects B input source, "DIRECT" (B port) or "CASCADE" (BCIN port)
      .PREADDINSEL("A"),                 // Selects input to pre-adder (A, B)
      .RND(48'h000000000000),            // Rounding Constant
      .USE_MULT("MULTIPLY"),             // Select multiplier usage (DYNAMIC, MULTIPLY, NONE)
      .USE_SIMD("ONE48"),                // SIMD selection (FOUR12, ONE48, TWO24)
      .USE_WIDEXOR("FALSE"),             // Use the Wide XOR function (FALSE, TRUE)
      .XORSIMD("XOR24_48_96"),           // Mode of operation for the Wide XOR (XOR12, XOR24_48_96)
      // Pattern Detector Attributes: Pattern Detection Configuration
      .AUTORESET_PATDET("NO_RESET"),     // NO_RESET, RESET_MATCH, RESET_NOT_MATCH
      .AUTORESET_PRIORITY("RESET"),      // Priority of AUTORESET vs. CEP (CEP, RESET).
      .MASK(48'h3fffffffffff),           // 48-bit mask value for pattern detect (1=ignore)
      .PATTERN(48'h000000000000),        // 48-bit pattern match for pattern detect
      .SEL_MASK("MASK"),                 // C, MASK, ROUNDING_MODE1, ROUNDING_MODE2
      .SEL_PATTERN("PATTERN"),           // Select pattern value (C, PATTERN)
      .USE_PATTERN_DETECT("NO_PATDET"),  // Enable pattern detect (NO_PATDET, PATDET)
      // Programmable Inversion Attributes: Specifies built-in programmable inversion on specific pins
      .IS_ALUMODE_INVERTED(4'b0000),     // Optional inversion for ALUMODE
      .IS_CARRYIN_INVERTED(1'b0),        // Optional inversion for CARRYIN
      .IS_CLK_INVERTED(1'b0),            // Optional inversion for CLK
      .IS_INMODE_INVERTED(5'b00000),     // Optional inversion for INMODE
      .IS_OPMODE_INVERTED(9'b000000000), // Optional inversion for OPMODE
      .IS_RSTALLCARRYIN_INVERTED(1'b0),  // Optional inversion for RSTALLCARRYIN
      .IS_RSTALUMODE_INVERTED(1'b0),     // Optional inversion for RSTALUMODE
      .IS_RSTA_INVERTED(1'b0),           // Optional inversion for RSTA
      .IS_RSTB_INVERTED(1'b0),           // Optional inversion for RSTB
      .IS_RSTCTRL_INVERTED(1'b0),        // Optional inversion for RSTCTRL
      .IS_RSTC_INVERTED(1'b0),           // Optional inversion for RSTC
      .IS_RSTD_INVERTED(1'b0),           // Optional inversion for RSTD
      .IS_RSTINMODE_INVERTED(1'b0),      // Optional inversion for RSTINMODE
      .IS_RSTM_INVERTED(1'b0),           // Optional inversion for RSTM
      .IS_RSTP_INVERTED(1'b0),           // Optional inversion for RSTP
      // Register Control Attributes: Pipeline Register Configuration
      .ACASCREG(1),                      // Number of pipeline stages between A/ACIN and ACOUT (0-2)
      .ADREG(1),                         // Pipeline stages for pre-adder (0-1)
      .ALUMODEREG(1),                    // Pipeline stages for ALUMODE (0-1)
      .AREG(1),                          // Pipeline stages for A (0-2)
      .BCASCREG(1),                      // Number of pipeline stages between B/BCIN and BCOUT (0-2)
      .BREG(1),                          // Pipeline stages for B (0-2)
      .CARRYINREG(1),                    // Pipeline stages for CARRYIN (0-1)
      .CARRYINSELREG(1),                 // Pipeline stages for CARRYINSEL (0-1)
      .CREG(1),                          // Pipeline stages for C (0-1)
      .DREG(1),                          // Pipeline stages for D (0-1)
      .INMODEREG(1),                     // Pipeline stages for INMODE (0-1)
      .MREG(1),                          // Multiplier pipeline stages (0-1)
      .OPMODEREG(1),                     // Pipeline stages for OPMODE (0-1)
      .PREG(1)                           // Number of pipeline stages for P (0-1)
   )
   DSP48E2_inst (
      // Cascade outputs: Cascade Ports
      .ACOUT(),                   // 30-bit output: A port cascade
      .BCOUT(),                   // 18-bit output: B cascade
      .CARRYCASCOUT(),     // 1-bit output: Cascade carry
      .MULTSIGNOUT(),       // 1-bit output: Multiplier sign cascade
      .PCOUT(y_data_out_cascade),                   // 48-bit output: Cascade output
      // Control outputs: Control Inputs/Status Bits
      .OVERFLOW(),             // 1-bit output: Overflow in add/acc
      .PATTERNBDETECT(), // 1-bit output: Pattern bar detect
      .PATTERNDETECT(),   // 1-bit output: Pattern detect
      .UNDERFLOW(),           // 1-bit output: Underflow in add/acc
      // Data outputs: Data Ports
      .CARRYOUT(),             // 4-bit output: Carry
      .P(y_data_out_fabric),                           // 48-bit output: Primary data
      .XOROUT(),                 // 8-bit output: XOR data
      // Cascade inputs: Cascade Ports
      .CLK(clk),
    .ACIN('b0),
    .BCIN('b0),
    .CARRYCASCIN('b0),
    .MULTSIGNIN('b0),
    .CARRYIN('b0),
    .D('b1),            // D unused thus can be tied to 1 to save routing resource. 1 is free tie-off. 0 consume route
    
      .PCIN(PCIN),                     // 48-bit input: P cascade
      // Control inputs: Control Inputs/Status Bits
      .ALUMODE(ALUMODE),               // 4-bit input: ALU control
      .CARRYINSEL(CARRYINSEL),         // 3-bit input: Carry select
      //.CLK(CLK),                       // 1-bit input: Clock
      .INMODE(INMODE),                 // 5-bit input: INMODE control
      .OPMODE(OPMODE),                 // 9-bit input: Operation mode
      // Data inputs: Data Ports
     .A(dsp_a_data),
    .B(dsp_b_data),
    .C(16'b0),                   // 48-bit input: C data
      //.CARRYIN(CARRYIN),               // 1-bit input: Carry-in
      // .D(D),                           // 27-bit input: D data
      // Reset/Clock Enable inputs
    .CEA1(1'b0),
    .CEA2(1'b1),
    .CEAD('b1),         // TODO: use use_multiplier to disable; but must be pipelined version
    .CEALUMODE('b0),
    .CEB1(1'b0),
    .CEB2(b_wen),         // TODO: use use_multiplier to disable; but must be pipelined version
    .CEC(use_c_data),
    .CEM('b1),          // TODO: use use_multiplier to disable; but must be pipelined version
    .CECARRYIN(1'b0),
    .CECTRL(1'b1),
    .CED(1'b0),
    .CEINMODE(1'b1),
    .CEP(1'b1),        // TODO: use use_multiplier to disable; but must be pipelined version
    .RSTA(1'b0),
    .RSTALLCARRYIN(1'b0),
    .RSTALUMODE(1'b0),
    .RSTB(1'b0),
    .RSTC(1'b0),
    .RSTCTRL(1'b0),
    .RSTD(1'b0),
    .RSTINMODE(1'b0),
    .RSTM(1'b0),
    .RSTP(1'b0)
    
   );

   // End of DSP48E2_inst instantiation
					
				
				


endmodule
